@@ -1,7 +1,7 @@
 /*
 	common: anything can happen here... frame reading, output, messages
 
-	copyright ?-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2020 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
@@ -14,14 +14,12 @@
 
 extern int stopped;
 extern int paused;
-
-/* Return width of terminal associated with given descriptor,
-   -1 when there is none. */
-int term_width(int fd);
+extern int muted;
 
 void print_header(mpg123_handle *);
 void print_header_compact(mpg123_handle *);
-void print_stat(mpg123_handle *fr, long offset, out123_handle *ao);
+void print_stat(mpg123_handle *fr, long offset, out123_handle *ao, int draw_bar
+,	struct parameter *param);
 void print_buf(const char* prefix, out123_handle *ao);
 void clear_stat();
 /* for control_generic */
