@@ -624,6 +624,7 @@ int decode_update(mpg123_handle *mh)
 	long native_rate;
 	int b;
 
+	mh->state_flags &= ~FRAME_DECODER_LIVE;
 	if(mh->num < 0)
 	{
 		if(!(mh->p.flags & MPG123_QUIET)) error("decode_update() has been called before reading the first MPEG frame! Internal programming error.");
@@ -690,6 +691,7 @@ int decode_update(mpg123_handle *mh)
 	debug3("done updating decoder structure with native rate %li and af.rate %li and down_sample %i", frame_freq(mh), mh->af.rate, mh->down_sample);
 
 	mh->decoder_change = 0;
+	mh->state_flags |= FRAME_DECODER_LIVE;
 	return 0;
 }
 
@@ -882,12 +884,15 @@ int attribute_align_arg mpg123_framebyframe_decode(mpg123_handle *mh, off_t *num
 	if(mh == NULL)    return MPG123_BAD_HANDLE;
 	if(mh->buffer.size < mh->outblock) return MPG123_NO_SPACE;
 
+	*audio = NULL;
 	*bytes = 0;
 	mh->buffer.fill = 0; /* always start fresh */
 	if(!mh->to_decode) return MPG123_OK;
 
 	if(num != NULL) *num = mh->num;
 	debug("decoding");
+	if(!(mh->state_flags & FRAME_DECODER_LIVE))
+		return MPG123_ERR;
 	decode_the_frame(mh);
 	mh->to_decode = mh->to_ignore = FALSE;
 	mh->buffer.p = mh->buffer.data;
@@ -967,7 +972,8 @@ int attribute_align_arg mpg123_decode_frame(mpg123_handle *mh, off_t *num, unsig
 			}
 			debug("decoding");
 
-			if(mh->decoder_change && decode_update(mh) < 0)
+			if( (mh->decoder_change && decode_update(mh) < 0)
+			||	!(mh->state_flags & FRAME_DECODER_LIVE) )
 				return MPG123_ERR;
 			decode_the_frame(mh);
 
@@ -1072,7 +1078,8 @@ int attribute_align_arg mpg123_decode(mpg123_handle *mh, const unsigned char *in
 				ret = MPG123_NO_SPACE;
 				goto decodeend;
 			}
-			if(mh->decoder_change && decode_update(mh) < 0)
+			if( (mh->decoder_change && decode_update(mh) < 0)
+			||	! (mh->state_flags & FRAME_DECODER_LIVE) )
 			{
 				ret = MPG123_ERR;
 				goto decodeend;
