@@ -1,7 +1,7 @@
 /*
 	out123_int: internal header for libout123
 
-	copyright ?-2016 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2021 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp (some traces left)
 */
@@ -14,6 +14,10 @@
 #include "config.h"
 #include "intsym.h"
 #include "abi_align.h"
+/* export DLL symbols */
+#if defined(WIN32) && defined(DYNAMIC_BUILD)
+#define BUILD_MPG123_DLL
+#endif
 #include "compat.h"
 #include "out123.h"
 #include "module.h"
