@@ -17,6 +17,10 @@
 #include "intsym.h"
 #include "compat.h"
 #include "abi_align.h"
+/* export DLL symbols */
+#if defined(WIN32) && defined(DYNAMIC_BUILD)
+#define BUILD_MPG123_DLL
+#endif
 #define SYN123_NO_LARGEFUNC
 #include "syn123.h"
 
